
/*
 *	SSSSSS  EEEEEE  RRRRRR   III   OOOOO        H     H
 *	S       E       R     R   I   O     O       H     H
 *	S       E       R     R   I   O     O       H     H
 *	SSSSSS  EEEEEE  RRRRRR    I   O     O       HHHHHHH
 *	     S  E       R   R     I   O     O       H     H
 *	     S  E       R    R    I   O     O  ..   H     H
 *	SSSSSS  EEEEEE  R     R  III   OOOOO   ..   H     H
 *
 *	serio.h -- serial I/O header file
 *
 *      Greg DeJager            1/25/89
 *
 *
 *      Adapted from design by:   Brian A. Berg
 *                                Berg Software Design
 *                                October 1988
 *
 *      minor adjustment for TC - b_smith 10/26/89
 *
 */

/* TC adjustment */
#define _dos_getvect(a)     getvect(a)
#define _dos_setvect(a,b)   setvect(a,b)

/* PIC 8259A parameters */
#define EOI		0x20		/* end-of-interrupt for PICTRL */
#define PICTRL		0x20
#define PICMSK		0x21

/* LBT invocation arguments: minumum and maximum values */
/*   argument 1: COM number */
#define COM_MIN		1		/* COM1 */
#define COM_MAX         3               /* COM3 */
#define COM_CNT		(COM_MAX-COM_MIN+1)
#define COM_DEF		1		/* DEFAULT: COM1 */
/*   argument 2: baud rate divisor (based on 1.8432 MHz crystal) */
#define BAUDIV_MIN	2		/* 56000 baud */
#define BAUDIV_MAX	2304		/* 50 baud */
#define BAUDIV_DEF      12              /* DEFAULT: 9600 baud */
#define BAUDIV_DEFB     96              /* DEFAULT: 1200 baud for 8250 */

/* macro to calculate baud rate using the baud rate divisor */
#define BAUDRATE(dvsr)	(115200L/((long)(dvsr)))

/* UART register definitions using UART base address */
#define RBR	(ubase+0)	/* Receive Buffer Register	(R  )(DLAB==0)*/
#define THR	(ubase+0)	/* Transmitter Holding Register	(  W)(DLAB==0)*/
#define IER	(ubase+1)	/* Interrupt Enable Register	(R/W)(DLAB==0)*/
#define IIR	(ubase+2)	/* Interrupt Ident. Register	(R  ) */
#define FCR     (ubase+2)       /* FIFO Control Register        (W  ) */
#define LCR     (ubase+3)       /* Line Control Register        (R/W) */
#define MCR	(ubase+4)	/* MODEM Control Register	(R/W) */
#define LSR	(ubase+5)	/* Line Status Register		(R  ) */
#define MSR	(ubase+6)	/* MODEM Status Register	(R/W) */
#define SCR	(ubase+7)	/* SCratch Register		(R/W) */
#define DLL	(ubase+0)	/* Divisor Latch (LSB)		(R/W)(DLAB==1)*/
#define DLM	(ubase+1)	/* Divisor Latch (MSB)		(R/W)(DLAB==1)*/

/* bit settings */
/*   16550 IIR  */
#define F_NOIP           0xC1             /* NO Interrupt Pending */
#define F_RLST           0xC6             /* Receiver Line STatus interrupt */
#define F_RDAV           0xC4             /* Received Data AVailable */
#define F_IIR_THRE       0xC2             /* Transmitter Holding Register Empty */
#define F_CHR_TIMEOUT    0xCC             /* Receiver FIFO Timeout interrupt */

/*  16450/8250 IIR: */
#define NOIP           0x01             /* NO Interrupt Pending */
#define RLST           0x06             /* Receiver Line STatus interrupt */
#define RDAV           0x04             /* Received Data AVailable */
#define IIR_THRE       0x02             /* Transmitter Holding Register Empty */
#define CHR_TIMEOUT    0x0C             /* Receiver FIFO Timeout interrupt */

/*   LSR and MSR: */
#define DR		1		/* bit 0: Data Ready */
#define OE		2		/* bit 1: Overrun Error */
#define PE		4		/* bit 2: Parity Error */
#define FE		8		/* bit 3: Framing Error */
#define BI		0x10		/* bit 4: Break Interrupt */
#define LSR_THRE	0x20		/* bit 5: Transmtr Holding Reg Empty */
#define TEMT		0x40		/* bit 6: Transmitter Empty */
#define CTS             0x10            /* bit 4: Clear To Send */

/* values for writing to IER */
#define IER_VAL1	0x05		/* RDA        & RLS */
#define IER_VAL2	0x07		/* RDA & THRE & RLS */

/* values for writing to MCR */
#define OUT2            0x08

/* values for writing to FCR */
#define CLR_FIFO        0x06            /* Clear Rx and Tx FIFOs */
#define FIFO_EN         0xC1            /* Enable FIFOs, set trigger to 14 */

/* data error ID numbers */
#define ENDPROG         1               /* keyboard hit */
#define FALSEINT        2               /* CPU int with IIR = 1 */
#define STATUSERR       3               /* Line Status Interrupt */
#define MISMATCH        4               /* Data mismatch */
#define RX_ERROR        5               /* RDAI with no DR indication */
#define TX_ERROR        6               /* THREI with no THRE indication */
#define IIR_ERROR       7               /* invalid IIR code */
#define TIMEOUT         8               /* Receiver FIFO Timeout Interrupt */
#define TIMEOUT_ERR     9               /* False Receiver FIFO Timeout */

/* macros */
#define rdIIR()		((UCHAR)inp(IIR))		/* read IIR */
#define rdLSR()		((UCHAR)inp(LSR))		/* read LSR */
#define rdMSR()         ((UCHAR)inp(MSR))               /* read MSR */
#define rdRBR()         ((UCHAR)inp(RBR))               /* read RBR */
#define wrTHR(val)      (outp(THR, val))                /* write THR */
#define wrIER(val)      (outp(IER, val))                /* write IER */
#define wrLCR(val)      (outp(LCR, val))                /* write LCR */
#define wrMCR(val)      (outp(MCR, val))                /* write MCR */
#define wrFCR(val)      (outp(FCR, val))                /* write FCR */

/* end of serio.h */
